@@ -181,6 +181,8 @@ enum
 #define DEFAULT_LATENCY    0
 #define DEFAULT_TOLERANCE  0
 #define DEFAULT_PLC        FALSE
+#define DEFAULT_DRAINABLE  TRUE
+#define DEFAULT_NEEDS_FORMAT  FALSE
 
 typedef struct _GstAudioDecoderContext
 {
@@ -262,6 +264,8 @@ struct _GstAudioDecoderPrivate
   GstClockTime latency;
   GstClockTime tolerance;
   gboolean plc;
+  gboolean drainable;
+  gboolean needs_format;
 
   /* pending serialized sink events, will be sent from finish_frame() */
   GList *pending_events;
@@ -394,6 +398,8 @@ gst_audio_decoder_init (GstAudioDecoder * dec, GstAudioDecoderClass * klass)
   dec->priv->latency = DEFAULT_LATENCY;
   dec->priv->tolerance = DEFAULT_TOLERANCE;
   dec->priv->plc = DEFAULT_PLC;
+  dec->priv->drainable = DEFAULT_DRAINABLE;
+  dec->priv->needs_format = DEFAULT_NEEDS_FORMAT;
 
   /* init state */
   gst_audio_decoder_reset (dec, TRUE);
@@ -417,6 +423,7 @@ gst_audio_decoder_reset (GstAudioDecoder * dec, gboolean full)
 
     gst_audio_info_clear (&dec->priv->ctx.info);
     memset (&dec->priv->ctx, 0, sizeof (dec->priv->ctx));
+    dec->priv->ctx.max_errors = GST_AUDIO_DECODER_MAX_ERRORS;
 
     if (dec->priv->taglist) {
       gst_tag_list_free (dec->priv->taglist);
@@ -998,6 +1005,7 @@ gst_audio_decoder_push_buffers (GstAudioDecoder * dec, gboolean force)
           break;
         } else if (ret == GST_FLOW_OK) {
           GST_LOG_OBJECT (dec, "frame at offset %d of length %d", offset, len);
+          g_assert (len);
           g_assert (offset + len <= av);
           priv->sync_flush = 0;
         } else {
@@ -1021,6 +1029,10 @@ gst_audio_decoder_push_buffers (GstAudioDecoder * dec, gboolean force)
     } else {
       if (!force)
         break;
+      if (!priv->drainable) {
+        priv->drained = TRUE;
+        break;
+      }
       buffer = NULL;
     }
 
@@ -1341,6 +1353,9 @@ gst_audio_decoder_chain (GstPad * pad, GstBuffer * buffer)
 
   dec = GST_AUDIO_DECODER (GST_PAD_PARENT (pad));
 
+  if (G_UNLIKELY (!GST_PAD_CAPS (pad) && dec->priv->needs_format))
+    goto not_negotiated;
+
   GST_LOG_OBJECT (dec,
       "received buffer of size %d with ts %" GST_TIME_FORMAT
       ", duration %" GST_TIME_FORMAT, GST_BUFFER_SIZE (buffer),
@@ -1378,6 +1393,15 @@ gst_audio_decoder_chain (GstPad * pad, GstBuffer * buffer)
   GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
   return ret;
+
+  /* ERRORS */
+not_negotiated:
+  {
+    GST_ELEMENT_ERROR (dec, CORE, NEGOTIATION, (NULL),
+        ("decoder not initialized"));
+    gst_buffer_unref (buffer);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 /* perform upstream byte <-> time conversion (duration, seeking)
@@ -2217,7 +2241,8 @@ gst_audio_decoder_get_delay (GstAudioDecoder * dec)
  * @num: max tolerated errors
  *
  * Sets numbers of tolerated decoder errors, where a tolerated one is then only
- * warned about, but more than tolerated will lead to fatal error.
+ * warned about, but more than tolerated will lead to fatal error.  Default
+ * is set to GST_AUDIO_DECODER_MAX_ERRORS.
  *
  * Since: 0.10.36
  */
@@ -2457,3 +2482,105 @@ gst_audio_decoder_get_tolerance (GstAudioDecoder * dec)
 
   return result;
 }
+
+/**
+ * gst_audio_decoder_set_drainable:
+ * @dec: a #GstAudioDecoder
+ * @enabled: new state
+ *
+ * Configures decoder drain handling.  If drainable, subclass might
+ * be handed a NULL buffer to have it return any leftover decoded data.
+ * Otherwise, it is not considered so capable and will only ever be passed
+ * real data.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.36
+ */
+void
+gst_audio_decoder_set_drainable (GstAudioDecoder * dec, gboolean enabled)
+{
+  g_return_if_fail (GST_IS_AUDIO_DECODER (dec));
+
+  GST_OBJECT_LOCK (dec);
+  dec->priv->drainable = enabled;
+  GST_OBJECT_UNLOCK (dec);
+}
+
+/**
+ * gst_audio_decoder_get_drainable:
+ * @dec: a #GstAudioDecoder
+ *
+ * Queries decoder drain handling.
+ *
+ * Returns: TRUE if drainable handling is enabled.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.36
+ */
+gboolean
+gst_audio_decoder_get_drainable (GstAudioDecoder * dec)
+{
+  gboolean result;
+
+  g_return_val_if_fail (GST_IS_AUDIO_DECODER (dec), 0);
+
+  GST_OBJECT_LOCK (dec);
+  result = dec->priv->drainable;
+  GST_OBJECT_UNLOCK (dec);
+
+  return result;
+}
+
+/**
+ * gst_audio_decoder_set_needs_format:
+ * @dec: a #GstAudioDecoder
+ * @enabled: new state
+ *
+ * Configures decoder format needs.  If enabled, subclass needs to be
+ * negotiated with format caps before it can process any data.  It will then
+ * never be handed any data before it has been configured.
+ * Otherwise, it might be handed data without having been configured and
+ * is then expected being able to do so either by default
+ * or based on the input data.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.36
+ */
+void
+gst_audio_decoder_set_needs_format (GstAudioDecoder * dec, gboolean enabled)
+{
+  g_return_if_fail (GST_IS_AUDIO_DECODER (dec));
+
+  GST_OBJECT_LOCK (dec);
+  dec->priv->needs_format = enabled;
+  GST_OBJECT_UNLOCK (dec);
+}
+
+/**
+ * gst_audio_decoder_get_needs_format:
+ * @dec: a #GstAudioDecoder
+ *
+ * Queries decoder required format handling.
+ *
+ * Returns: TRUE if required format handling is enabled.
+ *
+ * MT safe.
+ *
+ * Since: 0.10.36
+ */
+gboolean
+gst_audio_decoder_get_needs_format (GstAudioDecoder * dec)
+{
+  gboolean result;
+
+  g_return_val_if_fail (GST_IS_AUDIO_DECODER (dec), 0);
+
+  GST_OBJECT_LOCK (dec);
+  result = dec->priv->needs_format;
+  GST_OBJECT_UNLOCK (dec);
+
+  return result;
+}
