@@ -210,6 +210,7 @@ static void
 gst_discoverer_audio_info_finalize (GstDiscovererAudioInfo * info)
 {
   g_free (info->language);
+  gst_discoverer_stream_info_finalize ((GstDiscovererStreamInfo *) info);
 }
 
 static void
@@ -263,6 +264,7 @@ static void
 gst_discoverer_subtitle_info_finalize (GstDiscovererSubtitleInfo * info)
 {
   g_free (info->language);
+  gst_discoverer_stream_info_finalize ((GstDiscovererStreamInfo *) info);
 }
 
 static void
@@ -372,11 +374,13 @@ GstDiscovererInfo *
 gst_discoverer_info_copy (GstDiscovererInfo * ptr)
 {
   GstDiscovererInfo *ret;
-  GHashTable *stream_map = g_hash_table_new (g_direct_hash, NULL);
+  GHashTable *stream_map;
   GList *tmp;
 
   g_return_val_if_fail (ptr != NULL, NULL);
 
+  stream_map = g_hash_table_new (g_direct_hash, NULL);
+
   ret = gst_discoverer_info_new ();
 
   ret->uri = g_strdup (ptr->uri);
