@@ -34,40 +34,39 @@
 #include <gst/interfaces/xoverlay.h>
 #include <gst/interfaces/propertyprobe.h>
 
-#if !GTK_CHECK_VERSION (2, 17, 7)
-static void
-gtk_widget_get_allocation (GtkWidget * w, GtkAllocation * a)
-{
-  *a = w->allocation;
-}
-#endif
-
 static GtkWidget *video_window = NULL;
 static GstElement *sink = NULL;
 static gulong embed_xid = 0;
-static GdkGC *trans_gc = NULL;
+static GdkColor trans_color;
+static gboolean trans_color_set = FALSE;
 
 static void
 redraw_overlay (GtkWidget * widget)
 {
   GtkAllocation allocation;
   GdkWindow *window = gtk_widget_get_window (widget);
-  GtkStyle *style = gtk_widget_get_style (widget);
+  cairo_t *cr;
 
+  cr = gdk_cairo_create (window);
   gtk_widget_get_allocation (widget, &allocation);
-  gdk_draw_rectangle (window, style->white_gc, TRUE, 0, 0,
-      allocation.width, allocation.height);
 
-  if (trans_gc) {
+  cairo_set_source_rgb (cr, 1, 1, 1);
+  cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
+  cairo_fill (cr);
+
+  if (trans_color_set) {
     guint x, y;
     guint h = allocation.height * 0.75;
 
-    gdk_draw_rectangle (window, trans_gc, TRUE, 0, 0, allocation.width, h);
+    gdk_cairo_set_source_color (cr, &trans_color);
+    cairo_rectangle (cr, 0, 0, allocation.width, h);
+    cairo_fill (cr);
 
     for (y = h; y < allocation.height; y++) {
       for (x = 0; x < allocation.width; x++) {
         if (((x & 1) || (y & 1)) && (x & 1) != (y & 1)) {
-          gdk_draw_point (window, trans_gc, x, y);
+          cairo_move_to (cr, x, y);
+          cairo_paint (cr);
         }
       }
     }
@@ -82,7 +81,7 @@ handle_resize_cb (GtkWidget * widget, GdkEventConfigure * event, gpointer data)
 }
 
 static gboolean
-handle_expose_cb (GtkWidget * widget, GdkEventExpose * event, gpointer data)
+draw_cb (GtkWidget * widget, cairo_t * cr, gpointer data)
 {
   redraw_overlay (widget);
   return FALSE;
@@ -91,23 +90,15 @@ handle_expose_cb (GtkWidget * widget, GdkEventExpose * event, gpointer data)
 static void
 realize_cb (GtkWidget * widget, gpointer data)
 {
-#if GTK_CHECK_VERSION(2,18,0)
-  {
-    GdkWindow *window = gtk_widget_get_window (widget);
+  GdkWindow *window = gtk_widget_get_window (widget);
 
-    /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
-     * as well */
-    if (!gdk_window_ensure_native (window))
-      g_error ("Couldn't create native window needed for GstXOverlay!");
-  }
-#endif
+  /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
+   * as well */
+  if (!gdk_window_ensure_native (window))
+    g_error ("Couldn't create native window needed for GstXOverlay!");
 
-  {
-    GdkWindow *window = gtk_widget_get_window (video_window);
-
-    embed_xid = GDK_WINDOW_XID (window);
-    g_print ("Window realize: video window XID = %lu\n", embed_xid);
-  }
+  embed_xid = GDK_WINDOW_XID (window);
+  g_print ("Window realize: video window XID = %lu\n", embed_xid);
 }
 
 static void
@@ -127,18 +118,14 @@ msg_state_changed (GstBus * bus, GstMessage * message, GstPipeline * pipeline)
     /* When state of the pipeline changes to paused or playing we start updating scale */
     switch (GST_STATE_TRANSITION (old, new)) {
       case GST_STATE_CHANGE_READY_TO_PAUSED:{
-        GdkWindow *window = gtk_widget_get_window (video_window);
-
         g_object_get (G_OBJECT (sink), "colorkey", &color, NULL);
         if (color != -1) {
-          GdkColor trans_color = { 0,
-            (color & 0xff0000) >> 8,
-            (color & 0xff00),
-            (color & 0xff) << 8
-          };
-
-          trans_gc = gdk_gc_new (window);
-          gdk_gc_set_rgb_fg_color (trans_gc, &trans_color);
+          trans_color.red = (color & 0xff0000) >> 8;
+          trans_color.green = (color & 0xff00);
+          trans_color.blue = (color & 0xff) << 8;
+          trans_color_set = TRUE;
+        } else {
+          trans_color_set = FALSE;
         }
         handle_resize_cb (video_window, NULL, NULL);
         break;
@@ -155,7 +142,7 @@ window_closed (GtkWidget * widget, GdkEvent * event, gpointer user_data)
   GstElement *pipeline = user_data;
 
   g_print ("stopping\n");
-  gtk_widget_hide_all (widget);
+  gtk_widget_hide (widget);
   gst_element_set_state (pipeline, GST_STATE_NULL);
   gtk_main_quit ();
 }
@@ -185,8 +172,10 @@ main (int argc, char **argv)
   GstPropertyProbe *probe;
   GValueArray *arr;
 
+#if !GLIB_CHECK_VERSION (2, 31, 0)
   if (!g_thread_supported ())
     g_thread_init (NULL);
+#endif
 
   gst_init (&argc, &argv);
   gtk_init (&argc, &argv);
@@ -248,8 +237,8 @@ main (int argc, char **argv)
   video_window = gtk_drawing_area_new ();
   g_signal_connect (G_OBJECT (video_window), "configure-event",
       G_CALLBACK (handle_resize_cb), NULL);
-  g_signal_connect (G_OBJECT (video_window), "expose-event",
-      G_CALLBACK (handle_expose_cb), NULL);
+  g_signal_connect (G_OBJECT (video_window), "draw",
+      G_CALLBACK (draw_cb), NULL);
   g_signal_connect (video_window, "realize", G_CALLBACK (realize_cb), NULL);
   gtk_widget_set_double_buffered (video_window, FALSE);
   gtk_container_add (GTK_CONTAINER (window), video_window);
@@ -270,7 +259,7 @@ main (int argc, char **argv)
    * just set it directly here now (instead of waiting for a prepare-xwindow-id
    * element message in a sync bus handler and setting it there) */
   g_print ("setting XID %lu\n", embed_xid);
-  gst_x_overlay_set_xwindow_id (GST_X_OVERLAY (sink), embed_xid);
+  gst_x_overlay_set_window_handle (GST_X_OVERLAY (sink), embed_xid);
 
   g_idle_add (start_pipeline, pipeline);
   gtk_main ();
