@@ -1,8 +1,8 @@
 #ifndef _GST_PLUGINS_BASE__STDINT_H
 #define _GST_PLUGINS_BASE__STDINT_H 1
 #ifndef _GENERATED_STDINT_H
-#define _GENERATED_STDINT_H "gst-plugins-base 0.10.35.2"
-/* generated using gnu compiler gcc (Debian 4.6.2-5) 4.6.2 */
+#define _GENERATED_STDINT_H "gst-plugins-base 0.10.36"
+/* generated using gnu compiler gcc (Debian 4.6.2-12) 4.6.2 */
 #define _STDINT_HAVE_STDINT_H 1
 #include <stdint.h>
 #endif
